#include <assert.h>

#include "Interval.h"
#include "Matrix.h"
#include "Vector.h"
#include "igauss.h"

template <class Matrix>
void genTestMatrix(Matrix &A)
{
   unsigned int i,j;
   unsigned int n = A.rows();

   for(i=0;i<n;++i)
   {
      for(j=0;j<n;++j)
      {
         switch(i-j)
         {
            case 0:
               A(i,j)=2.0;
               break;
            case -1:
            case 1:
               A(i,j)=-1.0;
               break;
            default:
               A(i,j)=0.0;
               break;
         }
      }
   }
}

template <class Matrix, class Vector>
Vector operator *(const Matrix &A, const Vector &x) 
{
  assert(A.cols() == x.size());
  
  unsigned int n = A.rows();
  unsigned int m = A.cols();
  
  Vector y(n);
  
  typename Matrix::value_type sum;

  for (unsigned int i=0; i<n; i++) {
    sum = 0.0;
    for (unsigned int j=0; j<m; j++)
      sum += A(i,j) * x[j];
    y[i] = sum;
  }
  
  return y;
}


int main() 
{
  const unsigned int n=1000;
  
  Matrix<Interval> A(n,n);

  genTestMatrix(A);


  /*
  A(0,0) = 6.0;
  A(0,1) = 2.0;
  A(0,2) = 1.0;
  A(0,3) = -1.0;
  A(1,0) = 2.0;
  A(1,1) = 4.0;
  A(1,2) = 1.0;
  A(1,3) = 0.0;
  A(2,0) = 1.0;
  A(2,1) = 1.0;
  A(2,2) = 4.0;
  A(2,3) = -1.0;
  A(3,0) = -1.0;
  A(3,1) = 0.0;
  A(3,2) = -1.0;
  A(3,3) = 3.0;
  */
  
  /*
  A(0,0) = Interval(8.0);
  A(0,1) = Interval(1,4);
  A(0,2) = Interval(0,3);
  A(1,0) = Interval(1,2);
  A(1,1) = Interval(9.0);
  A(1,2) = Interval(0,6);
  A(2,0) = Interval(2,3);
  A(2,1) = Interval(2,5);
  A(2,2) = Interval(9.0);  
  */  

  //cout << "A: " << endl << A << endl;

  Matrix<Interval> B = A;

  Vector<Interval> b(n), x(n);

  for (unsigned int i=0; i<n; i++)
    b[i] = Interval(1e100, 1.001e100);
  
  /*
  b[0] = Interval(1.0, 1.1);
  b[1] = Interval(1.0);
  b[2] = Interval(1.0);  
  b[2] = Interval(1.0);  
  */

  //cout << "b: " << endl << b << endl;
  
  lu_factor(A);

  //cout << A << endl;
  
  lu_solve(A, b, x);  
  
  cout << "x: " << endl << x << endl;

  //cout << "A*x:" << endl << B*x << endl;
  

  return 0;
}

  



